@@ -139,8 +139,10 @@ module Control.Parallel.Strategies (
     NFData
   ) where
 
+#if !MIN_VERSION_base(4,8,0)
 import Data.Traversable
 import Control.Applicative
+#endif
 import Control.Parallel
 import Control.DeepSeq
 import Control.Monad
@@ -199,8 +201,15 @@ newtype Eval a = Eval (State# RealWorld -> (# State# RealWorld, a #))
 runEval :: Eval a -> a
 runEval (Eval x) = case x realWorld# of (# _, a #) -> a
 
+instance Functor Eval where
+  fmap = liftM
+
+instance Applicative Eval where
+  pure x = Eval $ \s -> (# s, x #)
+  (<*>)  = ap
+
 instance Monad Eval where
-  return x = Eval $ \s -> (# s, x #)
+  return = pure
   Eval x >>= k = Eval $ \s -> case x s of
                                 (# s', a #) -> case k a of
                                                       Eval f -> f s'
@@ -212,8 +221,15 @@ data Eval a = Done a
 runEval :: Eval a -> a
 runEval (Done x) = x
 
+instance Functor Eval where
+  fmap = liftM
+
+instance Applicative Eval where
+  pure = Done
+  (<*>) = ap
+
 instance Monad Eval where
-  return x = Done x
+  return = pure
   Done x >>= k = lazy (k x)   -- Note: pattern 'Done x' makes '>>=' strict
 
 {-# RULES "lazy Done" forall x . lazy (Done x) = Done x #-}
@@ -221,14 +237,6 @@ instance Monad Eval where
 #endif
 
 
-instance Functor Eval where
-  fmap = liftM
-
-instance Applicative Eval where
-  (<*>) = ap
-  pure  = return
-
-
 -- The Eval monad satisfies the monad laws.
 --
 -- (1) Left identity:
@@ -321,7 +329,7 @@ strat2 `dot` strat1 = strat2 . runEval . strat1
 evalSeq :: SeqStrategy a -> Strategy a
 evalSeq strat x = strat x `pseq` return x
 
--- | a name for @Control.Seq.Strategy@, for documetnation only.
+-- | A name for @Control.Seq.Strategy@, for documentation only.
 type SeqStrategy a = Control.Seq.Strategy a
 
 -- --------------------------------------------------------------------------
@@ -378,7 +386,7 @@ rdeepseq x = do rseq (rnf x); return x
 -- == rdeepseq
 
 -- | 'rpar' sparks its argument (for evaluation in parallel).
-rpar :: a -> Eval a
+rpar :: Strategy a
 #if __GLASGOW_HASKELL__ >= 702
 rpar  x = Eval $ \s -> spark# x s
 #else
@@ -507,6 +515,7 @@ parListWHNF xs = go xs `pseq` return xs
 -- more compositional counterpart; use RULES to do the specialisation.
 
 {-# NOINLINE [1] parList #-}
+{-# NOINLINE [1] rseq #-}
 {-# RULES
  "parList/rseq" parList rseq = parListWHNF
  #-}
@@ -792,7 +801,7 @@ Later, during work on the shared-memory implementation of
 parallelism in GHC, we discovered that the original formulation of
 Strategies had some problems, in particular it lead to space leaks
 and difficulties expressing speculative parallelism.  Details are in
-the paper /Runtime Support for Multicore Haskell/ <http://www.haskell.org/~simonmar/papers/multicore-ghc.pdf>.
+the paper /Runtime Support for Multicore Haskell/ <http://community.haskell.org/~simonmar/papers/multicore-ghc.pdf>.
 
 This module has been rewritten in version 2. The main change is to
 the 'Strategy a' type synonym, which was previously @a -> Done@ and
@@ -851,7 +860,7 @@ Version 3 introduced a major overhaul of the API, to match what is
 presented in the paper
 
   /Seq no More: Better Strategies for Parallel Haskell/
-  <http://www.haskell.org/~simonmar/papers/strategies.pdf>
+  <http://community.haskell.org/~simonmar/papers/strategies.pdf>
 
 The major differenes in the API are:
 
